@@ -1,5 +1,5 @@
 /* Tree-dumping functionality for intermediate representation.
-   Copyright (C) 1999-2013 Free Software Foundation, Inc.
+   Copyright (C) 1999-2015 Free Software Foundation, Inc.
    Written by Mark Mitchell <mark@codesourcery.com>
 
 This file is part of GCC.
@@ -22,7 +22,18 @@ along with GCC; see the file COPYING3.  If not see
 #include "system.h"
 #include "coretypes.h"
 #include "tm.h"
+#include "hash-set.h"
+#include "machmode.h"
+#include "vec.h"
+#include "double-int.h"
+#include "input.h"
+#include "alias.h"
+#include "symtab.h"
+#include "wide-int.h"
+#include "inchash.h"
+#include "real.h"
 #include "tree.h"
+#include "fixed-value.h"
 #include "splay-tree.h"
 #include "filenames.h"
 #include "tree-dump.h"
@@ -30,6 +41,7 @@ along with GCC; see the file COPYING3.  If not see
 #include "tree-iterator.h"
 #include "tree-pretty-print.h"
 #include "tree-cfg.h"
+#include "wide-int-print.h"
 
 static unsigned int queue (dump_info_p, const_tree, int);
 static void dump_index (dump_info_p, unsigned int);
@@ -394,9 +406,9 @@ dequeue_and_dump (dump_info_p di)
 		   (quals & TYPE_QUAL_CONST) ? 'c' : ' ',
 		   (quals & TYPE_QUAL_VOLATILE) ? 'v' : ' ',
 		   (quals & TYPE_QUAL_RESTRICT) ? 'r' : ' ',
-		   (quals & TYPE_QUAL_SHARED) ? 's' : ' ',
-		   (quals & TYPE_QUAL_STRICT) ? 't' : ' ',
-		   (quals & TYPE_QUAL_RELAXED) ? 'x' : ' ');
+		   (quals & TYPE_QUAL_UPC_SHARED) ? 's' : ' ',
+		   (quals & TYPE_QUAL_UPC_STRICT) ? 't' : ' ',
+		   (quals & TYPE_QUAL_UPC_RELAXED) ? 'x' : ' ');
 	  di->column += 14;
 	}
 
@@ -564,9 +576,8 @@ dequeue_and_dump (dump_info_p di)
       break;
 
     case INTEGER_CST:
-      if (TREE_INT_CST_HIGH (t))
-	dump_int (di, "high", TREE_INT_CST_HIGH (t));
-      dump_int (di, "low", TREE_INT_CST_LOW (t));
+      fprintf (di->stream, "int: ");
+      print_decs (t, di->stream);
       break;
 
     case STRING_CST:
