@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2013 Free Software Foundation, Inc.
+/* Copyright (C) 2001-2014 Free Software Foundation, Inc.
    This file is part of the UPC runtime Library.
    Written by Gary Funck <gary@intrepid.com>
    and Nenad Vukicevic <nenad@intrepid.com>
@@ -664,6 +664,16 @@ static void
 __upc_run_threads (upc_info_p u, int argc, char *argv[])
 {
   int thread_id;
+  int flag;
+
+  /* Set O_APPEND on stdout and stderr (see Brkeley UPC bug 2136).  */
+  flag = fcntl (STDOUT_FILENO, F_GETFL, 0);
+  if (flag >= 0)
+    (void) fcntl (STDOUT_FILENO, F_SETFL, flag | O_APPEND);
+  flag = fcntl (STDERR_FILENO, F_GETFL, 0);
+  if (flag >= 0)
+    (void) fcntl (STDERR_FILENO, F_SETFL, flag | O_APPEND);
+
   if (THREADS == 1)
     {
       __upc_affinity_set (u, 0);
